/*
 * i2sbus driver -- bus control routines
 *
 * Copyright 2006 Johannes Berg <johannes@sipsolutions.net>
 *
 * GPL v2, can be found in COPYING.
 */

#include <asm/io.h>
#include <linux/delay.h>
#include <asm/prom.h>
#include <asm/macio.h>
#include "i2sbus.h"

int i2sbus_control_init(struct macio_dev* dev, struct i2sbus_control **c)
{
	*c = kzalloc(sizeof(struct i2sbus_control), GFP_KERNEL);
	if (!*c)
		return -ENOMEM;

	if (of_address_to_resource(dev->ofdev.node, 0, &(*c)->rsrc))
		goto err;
	/* HACK HACK HACK */
	(*c)->rsrc.end = (*c)->rsrc.start + sizeof(struct i2s_control_regs);
	if (request_mem_region((*c)->rsrc.start,
				(*c)->rsrc.end - (*c)->rsrc.start + 1,
				"i2s control"))
		goto err;
	(*c)->controlregs = ioremap((*c)->rsrc.start,
				    (*c)->rsrc.end - (*c)->rsrc.start + 1);
	if (!(*c)->controlregs)
		goto release;

	printk(KERN_INFO "i2sbus: mapped i2s control registers\n");
	printk(KERN_DEBUG "i2sbus: control register contents:\n");
	printk(KERN_DEBUG "i2sbus:    fcr0 = 0x%x\n", in_le32(&(*c)->controlregs->fcr0));
	printk(KERN_DEBUG "i2sbus:    cell_control = 0x%x\n", in_le32(&(*c)->controlregs->cell_control));
	printk(KERN_DEBUG "i2sbus:    fcr2 = 0x%x\n", in_le32(&(*c)->controlregs->fcr2));
	printk(KERN_DEBUG "i2sbus:    fcr3 = 0x%x\n", in_le32(&(*c)->controlregs->fcr3));
	printk(KERN_DEBUG "i2sbus:    clock_control = 0x%x\n", in_le32(&(*c)->controlregs->clock_control));

	return 0;
 release:
	release_mem_region((*c)->rsrc.start,
			   (*c)->rsrc.end - (*c)->rsrc.start + 1);
 err:
	kfree(*c);
	*c = NULL;
	return -ENODEV;
}

void i2sbus_control_destroy(struct i2sbus_control *c)
{
	iounmap(c->controlregs);
	release_mem_region(c->rsrc.start, c->rsrc.end-c->rsrc.start+1);
	kfree(c);
	printk(KERN_DEBUG "i2sbus control destroyed\n");
}

void i2sbus_control_release(struct i2sbus_control *c)
{
	/* this is serialised externally */
	c->usecount--;
	if (c->usecount == 0)
		i2sbus_control_destroy(c);
}
