/*
 * i2sbus driver -- private definitions
 *
 * Copyright 2006 Johannes Berg <johannes@sipsolutions.net>
 *
 * GPL v2, can be found in COPYING.
 */
#ifndef __I2SBUS_H
#define __I2SBUS_H
#include <asm/dbdma.h>
#include <linux/interrupt.h>
#include <sound/pcm.h>
#include "i2sbus-interface.h"
#include "i2sbus-control.h"
/* FIXME */
#include "../soundbus.h"
/* should go away, see comment in i2sbus_add_dev */
#include <asm/prom.h>

struct i2sbus_control {
	volatile struct i2s_control_regs __iomem *controlregs;
	int usecount;
	struct resource rsrc;
};

#define MAX_DBDMA_COMMANDS	32

struct dbdma_command_mem {
	dma_addr_t bus_addr;
	dma_addr_t bus_cmd_start;
	struct dbdma_cmd *cmds;
	void *space;
	int size;
	u32 running:1;
};

struct pcm_info {
	int created:1, /* has this direction been created with alsa? */
	    active:1;  /* is this stream active? */
	/* runtime information */
	struct snd_pcm_substream *substream;
	int current_period;
	int periods;
	struct dbdma_command_mem dbdma_ring;
	volatile struct dbdma_regs __iomem *dbdma;
};

struct i2sbus_dev {
	struct soundbus_dev sound;
	struct macio_dev *macio;
	struct i2sbus_control *control;
	volatile struct i2s_interface_regs __iomem *intfregs;

	int resource_allocated; /* bitmask of resources */
	struct resource resources[3];
	int interrupts[3];

	/* info about currently active substreams */
	struct pcm_info out, in;
	snd_pcm_format_t format;
	unsigned int rate;
};

#define soundbus_dev_to_i2sbus_dev(sdev) \
		container_of(sdev, struct i2sbus_dev, sound)

/* pcm specific functions */
extern int
i2sbus_attach_codec(struct soundbus_dev *dev, struct snd_card *card,
		    struct codec_info *ci, void *data);
extern void
i2sbus_detach_codec(struct soundbus_dev *dev, void *data);
extern irqreturn_t
i2sbus_tx_intr(int irq, void *devid, struct pt_regs *regs);
extern irqreturn_t
i2sbus_rx_intr(int irq, void *devid, struct pt_regs *regs);

/* control specific functions */
extern int
i2sbus_control_init(struct macio_dev* dev, struct i2sbus_control **c);
extern void
i2sbus_control_destroy(struct i2sbus_control *c);
extern void
i2sbus_control_release(struct i2sbus_control *c);

#endif /* __I2SBUS_H */
